@@ -50,103 +50,74 @@
 #include "phaseexp.h"
 #include "bpf.h"
 #include "bpfb.h"
+#include "newamp1.h"
 
-void synth_one_frame(kiss_fft_cfg fft_inv_cfg, short buf[], MODEL *model, float Sn_[], float Pn[], int prede, float *de_mem, float gain);
+void synth_one_frame(int n_samp, codec2_fftr_cfg fftr_inv_cfg, short buf[], MODEL *model, float Sn_[], float Pn[], int prede, float *de_mem, float gain);
 void print_help(const struct option *long_options, int num_opts, char* argv[]);
 
+#define N_SAMP n_samp  /* quick fix for run time sample rate selection */
 
 /*---------------------------------------------------------------------------*\
-                                                                          
-				MAIN                                        
-                                                                         
+
+				MAIN
+
 \*---------------------------------------------------------------------------*/
 
 int main(int argc, char *argv[])
 {
-    FILE *fout = NULL;	/* output speech file                    */
-    FILE *fin;		/* input speech file                     */
-    short buf[N];	/* input/output buffer                   */
-    float buf_float[N];
-    float buf_float_bpf[N];
-    float Sn[M];	/* float input speech samples            */
-    float Sn_pre[N];	/* pre-emphasised input speech samples   */
-    COMP  Sw[FFT_ENC];	/* DFT of Sn[]                           */
-    kiss_fft_cfg  fft_fwd_cfg;
-    kiss_fft_cfg  fft_inv_cfg;
-    float w[M];	        /* time domain hamming window            */
-    COMP  W[FFT_ENC];	/* DFT of w[]                            */
-    MODEL model;
-    float Pn[2*N];	/* trapezoidal synthesis window          */
-    float Sn_[2*N];	/* synthesised speech */
-    int   i,m;		/* loop variable                         */
-    int   frames;
-    float prev_Wo, prev__Wo, prev_uq_Wo;
-    float pitch;
-    char  out_file[MAX_STR];
-    char  ampexp_arg[MAX_STR];
-    char  phaseexp_arg[MAX_STR];
-    float snr;
-    float sum_snr;
-    int orderi;
+
+    int Fs = 8000;
+    int set_fs;
+
     int lpc_model = 0, order = LPC_ORD;
     int lsp = 0, lspd = 0, lspvq = 0;
     int lspres = 0;
     int lspjvm = 0, lspjnd = 0, lspmel = 0, lspmelvq = 0;
     #ifdef __EXPERIMENTAL__
-    int lspanssi = 0, 
+    int lspanssi = 0,
     #endif
     int prede = 0;
-    float pre_mem = 0.0, de_mem = 0.0;
-    float ak[order];
-    COMP  Sw_[FFT_ENC];
-    COMP  Ew[FFT_ENC]; 
- 
-    int phase0 = 0;
-    float ex_phase[MAX_AMP+1];
-
     int   postfilt;
-    float bg_est = 0.0;
-
     int   hand_voicing = 0, phaseexp = 0, ampexp = 0, hi = 0, simlpcpf = 0, lspmelread = 0;
     int   lpcpf = 0;
     FILE *fvoicing = 0;
     FILE *flspmel = 0;
-
-    MODEL prev_model;
     int dec;
     int decimate = 1;
-    float lsps[order];
-    float e, prev_e;
-    int   lsp_indexes[order];
-    float lsps_[order];
-    float Woe_[2];
-
-    float lsps_dec[4][LPC_ORD], e_dec[4], weight, weight_inc, ak_dec[4][LPC_ORD];
-    MODEL model_dec[4], prev_model_dec;
-    float prev_lsps_dec[order], prev_e_dec;
-
-    void *nlp_states;
-    float hpf_states[2];
+    int   amread, Woread, pahw;
+    int   awread;
+    int   hmread;
+    int   phase0 = 0;
     int   scalar_quant_Wo_e = 0;
     int   scalar_quant_Wo_e_low = 0;
     int   vector_quant_Wo_e = 0;
     int   dump_pitch_e = 0;
-    FILE *fjvm = NULL;
-    #ifdef DUMP
-    int   dump;
-    #endif
-    struct PEXP *pexp = NULL;
-    struct AEXP *aexp = NULL;
     float gain = 1.0;
     int   bpf_en = 0;
     int   bpfb_en = 0;
-    float bpf_buf[BPF_N+N];
-    float lspmelvq_mse = 0.0;
-    int   amread;
-    FILE *fam;
-
+    FILE *fam = NULL, *fWo = NULL;
+    FILE *faw = NULL;
+    FILE *fhm = NULL;
+    FILE *fjvm = NULL;
+    FILE *flspEWov = NULL;
+    FILE *ften_ms_centre = NULL;
+    #ifdef DUMP
+    int   dump;
+    #endif
+    char  ampexp_arg[MAX_STR];
+    char  phaseexp_arg[MAX_STR];
+    char  out_file[MAX_STR];
+    FILE *fout = NULL;	/* output speech file */
+    int   mel_resampling = 0;
+    int   K = 20;
+    float framelength_s = N_S;
+    int   lspEWov = 0;
+    int   ten_ms_centre = 0;
+    
     char* opt_string = "ho:";
     struct option long_options[] = {
+        { "Fs", required_argument, &set_fs, 1 },
+        { "mel", required_argument, &mel_resampling, 1 },
         { "lpc", required_argument, &lpc_model, 1 },
         { "lspjnd", no_argument, &lspjnd, 1 },
         { "lspmel", no_argument, &lspmel, 1 },
@@ -179,6 +150,13 @@ int main(int argc, char *argv[])
         { "bpf", no_argument, &bpf_en, 1 },
         { "bpfb", no_argument, &bpfb_en, 1 },
         { "amread", required_argument, &amread, 1 },
+        { "hmread", required_argument, &hmread, 1 },
+        { "awread", required_argument, &awread, 1 },
+        { "Woread", required_argument, &Woread, 1 },
+        { "pahw", required_argument, &pahw, 1 },
+        { "lspEWov", required_argument, &lspEWov, 1 },
+        { "ten_ms_centre", required_argument, &ten_ms_centre, 1 },
+        { "framelength_s", required_argument, NULL, 0 },
         #ifdef DUMP
         { "dump", required_argument, &dump, 1 },
         #endif
@@ -186,35 +164,6 @@ int main(int argc, char *argv[])
         { NULL, no_argument, NULL, 0 }
     };
     int num_opts=sizeof(long_options)/sizeof(struct option);
-    COMP Aw[FFT_ENC];
-
-    for(i=0; i<M; i++) {
-	Sn[i] = 1.0;
-	Sn_pre[i] = 1.0;
-    }
-    for(i=0; i<2*N; i++)
-	Sn_[i] = 0;
-
-    prev_uq_Wo = prev_Wo = prev__Wo = TWO_PI/P_MAX;
-
-    prev_model.Wo = TWO_PI/P_MIN;
-    prev_model.L = floor(PI/prev_model.Wo);
-    for(i=1; i<=prev_model.L; i++) {
-	prev_model.A[i] = 0.0;
-	prev_model.phi[i] = 0.0;
-    }
-    for(i=1; i<=MAX_AMP; i++) {
-	//ex_phase[i] = (PI/3)*(float)rand()/RAND_MAX;
-	ex_phase[i] = 0.0;
-    }
-    e = prev_e = 1;
-    hpf_states[0] = hpf_states[1] = 0.0;
-
-    nlp_states = nlp_create(M);
-
-    if (argc < 2) {
-        print_help(long_options, num_opts, argv);
-    }
 
     /*----------------------------------------------------------------*\
 
@@ -222,24 +171,31 @@ int main(int argc, char *argv[])
 
     \*----------------------------------------------------------------*/
 
+    if (argc < 2) {
+        print_help(long_options, num_opts, argv);
+    }
+
     while(1) {
         int option_index = 0;
-        int opt = getopt_long(argc, argv, opt_string, 
+        int opt = getopt_long(argc, argv, opt_string,
                     long_options, &option_index);
         if (opt == -1)
             break;
         switch (opt) {
          case 0:
-            if(strcmp(long_options[option_index].name, "lpc") == 0) {
-                orderi = atoi(optarg);
-                if((orderi < 4) || (orderi > order)) {
-                    fprintf(stderr, "Error in LPC order (4 to %d): %s\n", order, optarg);
+            if(strcmp(long_options[option_index].name, "Fs") == 0) {
+                Fs= atoi(optarg);
+                if((Fs != 8000) && (Fs != 16000)) {
+                    fprintf(stderr, "Error Fs must be 8000 or 16000\n");
                     exit(1);
                 }
-                order = orderi;
+            } else if(strcmp(long_options[option_index].name, "mel") == 0) {
+                K = atoi(optarg);
+            } else if(strcmp(long_options[option_index].name, "lpc") == 0) {
+                order = atoi(optarg);
             #ifdef DUMP
             } else if(strcmp(long_options[option_index].name, "dump") == 0) {
-                if (dump) 
+                if (dump)
 	            dump_on(optarg);
             #endif
             } else if(strcmp(long_options[option_index].name, "lsp") == 0
@@ -247,7 +203,7 @@ int main(int argc, char *argv[])
                   || strcmp(long_options[option_index].name, "lspvq") == 0) {
 	        assert(order == LPC_ORD);
             } else if(strcmp(long_options[option_index].name, "dec") == 0) {
-               
+
                 decimate = atoi(optarg);
 	        if ((decimate != 2) && (decimate != 3) && (decimate != 4)) {
 	            fprintf(stderr, "Error in --dec, must be 2, 3, or 4\n");
@@ -262,7 +218,7 @@ int main(int argc, char *argv[])
                     printf("needs --lpc [order] to resample amplitudes when using --dec\n");
                     exit(1);
                 }
- 
+
             } else if(strcmp(long_options[option_index].name, "hand_voicing") == 0) {
 	        if ((fvoicing = fopen(optarg,"rt")) == NULL) {
 	            fprintf(stderr, "Error opening voicing file: %s: %s.\n",
@@ -275,12 +231,30 @@ int main(int argc, char *argv[])
 		        optarg, strerror(errno));
                     exit(1);
                 }
+            } else if(strcmp(long_options[option_index].name, "Woread") == 0) {
+	        if ((fWo = fopen(optarg,"rb")) == NULL) {
+	            fprintf(stderr, "Error opening float Wo file: %s: %s.\n",
+		        optarg, strerror(errno));
+                    exit(1);
+                }
             } else if(strcmp(long_options[option_index].name, "amread") == 0) {
 	        if ((fam = fopen(optarg,"rb")) == NULL) {
 	            fprintf(stderr, "Error opening float Am file: %s: %s.\n",
 		        optarg, strerror(errno));
                     exit(1);
                 }
+            } else if(strcmp(long_options[option_index].name, "hmread") == 0) {
+	        if ((fhm = fopen(optarg,"rb")) == NULL) {
+	            fprintf(stderr, "Error opening float Hm file: %s: %s.\n",
+		        optarg, strerror(errno));
+                    exit(1);
+                }
+            } else if(strcmp(long_options[option_index].name, "awread") == 0) {
+	        if ((faw = fopen(optarg,"rb")) == NULL) {
+	            fprintf(stderr, "Error opening float Aw file: %s: %s.\n",
+                            optarg, strerror(errno));
+                    exit(1);
+                }
 	    } else if(strcmp(long_options[option_index].name, "dump_pitch_e") == 0) {
 	        if ((fjvm = fopen(optarg,"wt")) == NULL) {
 	            fprintf(stderr, "Error opening pitch & energy dump file: %s: %s.\n",
@@ -293,6 +267,52 @@ int main(int argc, char *argv[])
 		strcpy(ampexp_arg, optarg);
 	    } else if(strcmp(long_options[option_index].name, "gain") == 0) {
 		gain = atof(optarg);
+	    } else if(strcmp(long_options[option_index].name, "framelength_s") == 0) {
+		framelength_s = atof(optarg);
+	    } else if(strcmp(long_options[option_index].name, "pahw") == 0) {
+
+                /* set up a bunch of arguments instead of having to enter them on cmd line every time */
+
+                phase0 = postfilt = amread = hmread = Woread = 1;
+                char file_name[MAX_STR];
+                sprintf(file_name, "%s_am.out", optarg);
+                fprintf(stderr, "reading %s", file_name);
+	        if ((fam = fopen(file_name,"rb")) == NULL) {
+	            fprintf(stderr, "Error opening float Am file: %s: %s.\n",
+		        file_name, strerror(errno));
+                    exit(1);
+                }
+                sprintf(file_name, "%s_hm.out", optarg);
+                fprintf(stderr, " %s", file_name);
+	        if ((fhm = fopen(file_name,"rb")) == NULL) {
+	            fprintf(stderr, "Error opening float Hm file: %s: %s.\n",
+		        file_name, strerror(errno));
+                    exit(1);
+                }
+                sprintf(file_name, "%s_Wo.out", optarg);
+                fprintf(stderr, " %s\n", file_name);
+ 	        if ((fWo = fopen(file_name,"rb")) == NULL) {
+	            fprintf(stderr, "Error opening float Wo file: %s: %s.\n",
+		        file_name, strerror(errno));
+                    exit(1);
+                }
+	    } else if(strcmp(long_options[option_index].name, "lspEWov") == 0) {
+                /* feature file for deep learning experiments */
+                lpc_model = 1; phase0 = 1;
+	        if ((flspEWov = fopen(optarg,"wb")) == NULL) {
+	            fprintf(stderr, "Error opening lspEWov float file: %s: %s\n",
+		        optarg, strerror(errno));
+                    exit(1);
+                }                 
+	    } else if(strcmp(long_options[option_index].name, "ten_ms_centre") == 0) {
+                /* dump 10ms of audio centred on analysis frame to check time alignment with 
+                   16 kHz source audio */
+                ten_ms_centre = 1;
+	        if ((ften_ms_centre = fopen(optarg,"wb")) == NULL) {
+	            fprintf(stderr, "Error opening ten_ms_centre short file: %s: %s\n",
+		        optarg, strerror(errno));
+                    exit(1);
+                }                 
 	    } else if(strcmp(long_options[option_index].name, "rate") == 0) {
                 if(strcmp(optarg,"3200") == 0) {
 	            lpc_model = 1;
@@ -363,6 +383,7 @@ int main(int argc, char *argv[])
 
     /* Input file */
 
+    FILE *fin;		/* input speech file                     */
     if (strcmp(argv[optind], "-")  == 0) fin = stdin;
     else if ((fin = fopen(argv[optind],"rb")) == NULL) {
 	fprintf(stderr, "Error opening input speech file: %s: %s.\n",
@@ -370,6 +391,88 @@ int main(int argc, char *argv[])
 	exit(1);
     }
 
+    C2CONST c2const = c2const_create(Fs, framelength_s);
+    int   n_samp = c2const.n_samp;
+    int   m_pitch = c2const.m_pitch;
+
+    short buf[N_SAMP];	/* input/output buffer                   */
+    float buf_float[N_SAMP];
+    float Sn[m_pitch];	/* float input speech samples            */
+    float Sn_pre[m_pitch];	/* pre-emphasised input speech samples   */
+    COMP  Sw[FFT_ENC];	/* DFT of Sn[]                           */
+    codec2_fft_cfg  fft_fwd_cfg;
+    codec2_fftr_cfg  fftr_fwd_cfg;
+    codec2_fftr_cfg  fftr_inv_cfg;
+    float w[m_pitch];	        /* time domain hamming window            */
+    float W[FFT_ENC];	/* DFT of w[]                            */
+    MODEL model;
+    float Pn[2*N_SAMP];	/* trapezoidal synthesis window          */
+    float Sn_[2*N_SAMP];	/* synthesised speech */
+    int   i,m;		/* loop variable                         */
+    int   frames;
+    float prev_f0;
+    float pitch;
+    float snr;
+    float sum_snr;
+
+    float pre_mem = 0.0, de_mem = 0.0;
+    float ak[1+order];
+    // COMP  Sw_[FFT_ENC];
+    // COMP  Ew[FFT_ENC];
+
+    float ex_phase[MAX_AMP+1];
+
+    float bg_est = 0.0;
+
+
+    MODEL prev_model;
+    float lsps[order];
+    float e, prev_e;
+    int   lsp_indexes[order];
+    float lsps_[order];
+    float Woe_[2];
+
+    float lsps_dec[4][order], e_dec[4], weight, weight_inc, ak_dec[4][order];
+    MODEL model_dec[4], prev_model_dec;
+    float prev_lsps_dec[order], prev_e_dec;
+
+    void *nlp_states;
+    float hpf_states[2];
+    #if 0
+    struct PEXP *pexp = NULL;
+    struct AEXP *aexp = NULL;
+    #endif
+    float bpf_buf[BPF_N+N_SAMP];
+    float lspmelvq_mse = 0.0;
+
+    COMP Aw[FFT_ENC];
+    COMP H[MAX_AMP];
+
+ 
+    for(i=0; i<m_pitch; i++) {
+	Sn[i] = 1.0;
+	Sn_pre[i] = 1.0;
+    }
+    for(i=0; i<2*N_SAMP; i++)
+	Sn_[i] = 0;
+
+    prev_f0 = 1/P_MAX_S;
+
+    prev_model.Wo = c2const.Wo_max;
+    prev_model.L = floor(PI/prev_model.Wo);
+    for(i=1; i<=prev_model.L; i++) {
+	prev_model.A[i] = 0.0;
+	prev_model.phi[i] = 0.0;
+    }
+    for(i=1; i<=MAX_AMP; i++) {
+	//ex_phase[i] = (PI/3)*(float)rand()/RAND_MAX;
+	ex_phase[i] = 0.0;
+    }
+    e = prev_e = 1;
+    hpf_states[0] = hpf_states[1] = 0.0;
+
+    nlp_states = nlp_create(&c2const);
+
     ex_phase[0] = 0;
     Woe_[0] = Woe_[1] = 1.0;
 
@@ -381,15 +484,23 @@ int main(int argc, char *argv[])
 
     /* Initialise ------------------------------------------------------------*/
 
-    fft_fwd_cfg = kiss_fft_alloc(FFT_ENC, 0, NULL, NULL); /* fwd FFT,used in several places   */
-    fft_inv_cfg = kiss_fft_alloc(FFT_DEC, 1, NULL, NULL); /* inverse FFT, used just for synth */
-    make_analysis_window(fft_fwd_cfg, w, W);
-    make_synthesis_window(Pn);
+    fft_fwd_cfg = codec2_fft_alloc(FFT_ENC, 0, NULL, NULL);   /* fwd FFT,used in several places   */
+    fftr_fwd_cfg = codec2_fftr_alloc(FFT_ENC, 0, NULL, NULL); /* fwd FFT,used in several places   */
+    fftr_inv_cfg = codec2_fftr_alloc(FFT_DEC, 1, NULL, NULL); /* inverse FFT, used just for synth */
+    codec2_fft_cfg phase_fft_fwd_cfg = codec2_fft_alloc(NEWAMP1_PHASE_NFFT, 0, NULL, NULL);
+    codec2_fft_cfg phase_fft_inv_cfg = codec2_fft_alloc(NEWAMP1_PHASE_NFFT, 1, NULL, NULL);
+
+    make_analysis_window(&c2const, fft_fwd_cfg, w, W);
+    make_synthesis_window(&c2const, Pn);
     quantise_init();
+
+    /* disabled for now while we convert to runtime n_samp */
+    #if 0
     if (phaseexp)
 	pexp = phase_experiment_create();
     if (ampexp)
 	aexp = amp_experiment_create();
+    #endif
 
     if (bpfb_en)
         bpf_en = 1;
@@ -404,10 +515,21 @@ int main(int argc, char *argv[])
     prev_e_dec = 1;
     for(m=1; m<=MAX_AMP; m++)
 	prev_model_dec.A[m] = 0.0;
-    prev_model_dec.Wo = TWO_PI/P_MAX;
+    prev_model_dec.Wo = c2const.Wo_min;
     prev_model_dec.L = PI/prev_model_dec.Wo;
     prev_model_dec.voiced = 0;
 
+    /* mel resampling experiments */
+
+    float rate_K_sample_freqs_kHz[K];
+    if (mel_resampling) {
+        float mel_start = ftomel(100); 
+        float mel_end = ftomel(0.95*Fs/2);
+        mel_sample_freqs_kHz(rate_K_sample_freqs_kHz, K, mel_start, mel_end);
+        //for(i=0; i<K; i++)
+        //    fprintf(stderr, "%d %f\n", i, rate_K_sample_freqs_kHz[i]);
+    }
+
     /*----------------------------------------------------------------* \
 
                             Main Loop
@@ -416,44 +538,44 @@ int main(int argc, char *argv[])
 
     frames = 0;
     sum_snr = 0;
-    while(fread(buf,sizeof(short),N,fin)) {
+    while(fread(buf,sizeof(short),N_SAMP,fin)) {
 	frames++;
 
-	for(i=0; i<N; i++)
+	for(i=0; i<N_SAMP; i++)
 	    buf_float[i] = buf[i];
 
 	/* optionally filter input speech */
-	
+
         if (prede) {
-           pre_emp(Sn_pre, buf_float, &pre_mem, N);
-           for(i=0; i<N; i++)
+           pre_emp(Sn_pre, buf_float, &pre_mem, N_SAMP);
+           for(i=0; i<N_SAMP; i++)
                 buf_float[i] = Sn_pre[i];
         }
-         
+
         if (bpf_en) {
-            /* filter input speech to create buf_float_bpf[], this is fed to the 
+            /* filter input speech to create buf_float_bpf[], this is fed to the
                LPC modelling.  Unfiltered speech in in buf_float[], which is
                delayed to match that of the BPF */
 
             /* BPF speech */
 
             for(i=0; i<BPF_N; i++)
-                bpf_buf[i] =  bpf_buf[N+i];
-            for(i=0; i<N; i++)
+                bpf_buf[i] =  bpf_buf[N_SAMP+i];
+            for(i=0; i<N_SAMP; i++)
                 bpf_buf[BPF_N+i] = buf_float[i];
             if (bpfb_en)
-                inverse_filter(&bpf_buf[BPF_N], bpfb, N, buf_float, BPF_N);
+                inverse_filter(&bpf_buf[BPF_N], bpfb, N_SAMP, buf_float, BPF_N);
             else
-                inverse_filter(&bpf_buf[BPF_N], bpf, N, buf_float, BPF_N);
+                inverse_filter(&bpf_buf[BPF_N], bpf, N_SAMP, buf_float, BPF_N);
         }
 
         /* shift buffer of input samples, and insert new samples */
 
-	for(i=0; i<M-N; i++) {
-	    Sn[i] = Sn[i+N];
+	for(i=0; i<m_pitch-N_SAMP; i++) {
+	    Sn[i] = Sn[i+N_SAMP];
 	}
-	for(i=0; i<N; i++) {
-	    Sn[i+M-N] = buf_float[i];
+	for(i=0; i<N_SAMP; i++) {
+	    Sn[i+m_pitch-N_SAMP] = buf_float[i];
         }
 
 	/*------------------------------------------------------------*\
@@ -462,17 +584,18 @@ int main(int argc, char *argv[])
 
 	\*------------------------------------------------------------*/
 
-	nlp(nlp_states,Sn,N,P_MIN,P_MAX,&pitch,Sw,W,&prev_uq_Wo);
+        nlp(nlp_states, Sn, N_SAMP, &pitch, Sw, W, &prev_f0);
 	model.Wo = TWO_PI/pitch;
 
-	dft_speech(fft_fwd_cfg, Sw, Sn, w);
-	two_stage_pitch_refinement(&model, Sw);
+        dft_speech(&c2const, fft_fwd_cfg, Sw, Sn, w);
+	two_stage_pitch_refinement(&c2const, &model, Sw);
 	estimate_amplitudes(&model, Sw, W, 1);
- 
-        #ifdef DUMP 
-	dump_Sn(Sn); dump_Sw(Sw); dump_model(&model);
+
+        #ifdef DUMP
+        dump_Sn(m_pitch, Sn); dump_Sw(Sw); dump_model(&model);
         #endif
 
+        #if 0
 	if (ampexp)
 	    amp_experiment(aexp, &model, ampexp_arg);
 
@@ -485,7 +608,8 @@ int main(int argc, char *argv[])
 	    dump_phase_(&model.phi[0], model.L);
             #endif
 	}
-	
+        #endif
+
 	if (hi) {
 	    int m;
 	    for(m=1; m<model.L/2; m++)
@@ -494,6 +618,18 @@ int main(int argc, char *argv[])
 		model.A[m] = 0.0;
 	}
 
+	/*------------------------------------------------------------*\
+
+	                     Mel scale resampling
+
+	\*------------------------------------------------------------*/
+
+        if (mel_resampling) {
+            float rate_K_vec[K];
+            resample_const_rate_f(&c2const, &model, rate_K_vec, rate_K_sample_freqs_kHz, K);
+            resample_rate_L(&c2const, &model, rate_K_vec, rate_K_sample_freqs_kHz, K);
+        }
+
 	/*------------------------------------------------------------*\
 
                             Zero-phase modelling
@@ -501,24 +637,17 @@ int main(int argc, char *argv[])
 	\*------------------------------------------------------------*/
 
 	if (phase0) {
-	    float Wn[M];		        /* windowed speech samples */
-	    float Rk[order+1];                  /* autocorrelation coeffs  */
-            COMP a[FFT_ENC];	                
-
             #ifdef DUMP
 	    dump_phase(&model.phi[0], model.L);
             #endif
 
-	    /* find aks here, these are overwritten if LPC modelling is enabled */
-
-            for(i=0; i<M; i++)
-                Wn[i] = Sn[i]*w[i];
-	    autocorrelate(Wn,Rk,M,order);
-	    levinson_durbin(Rk,ak,order);
-
 	    /* determine voicing */
 
-	    snr = est_voicing_mbe(&model, Sw, W, Sw_, Ew);
+	    #if 0
+            snr = est_voicing_mbe(&c2const, &model, Sw, W, Sw_, Ew);
+            #else
+	    snr = est_voicing_mbe(&c2const, &model, Sw, W);
+            #endif
 
 	    if (dump_pitch_e)
 		fprintf(fjvm, "%f %f %d ", model.Wo, snr, model.voiced);
@@ -526,8 +655,8 @@ int main(int argc, char *argv[])
 	    //printf("snr %3.2f v: %d Wo: %f prev_Wo: %f\n", snr, model.voiced,
 	    //	   model.Wo, prev_uq_Wo);
             #ifdef DUMP
-	    dump_Sw_(Sw_);
-	    dump_Ew(Ew);
+	    //dump_Sw_(Sw_);
+	    //dump_Ew(Ew);
 	    dump_snr(snr);
             #endif
 
@@ -535,47 +664,36 @@ int main(int argc, char *argv[])
 
 	    for(i=0; i<=MAX_AMP; i++)
 	    	model.phi[i] = 0;
-	    
-            /* Determine DFT of A(exp(jw)), which is needed for phase0 model when
-               LPC is not used, e.g. indecimate=1 (10ms) frames with no LPC */
-
-            for(i=0; i<FFT_ENC; i++) {
-                a[i].real = 0.0;
-                a[i].imag = 0.0; 
-            }
-
-            for(i=0; i<=order; i++)
-                a[i].real = ak[i];
-            kiss_fft(fft_fwd_cfg, (kiss_fft_cpx *)a, (kiss_fft_cpx *)Aw);
 
 	    if (hand_voicing) {
-		fscanf(fvoicing,"%d\n",&model.voiced);
+		int ret = fscanf(fvoicing,"%d\n",&model.voiced);
+                assert(ret == 1);
 	    }
 	}
 
 	/*------------------------------------------------------------*\
 
-	        LPC model amplitudes and LSP quantisation 
+	        LPC model amplitudes and LSP quantisation
 
 	\*------------------------------------------------------------*/
 
 	if (lpc_model) {
-	    
-            e = speech_to_uq_lsps(lsps, ak, Sn, w, order);
-            for(i=0; i<LPC_ORD; i++)
-                lsps_[i] = lsps[i];
 
+            e = speech_to_uq_lsps(lsps, ak, Sn, w, m_pitch, order);
+            for(i=0; i<order; i++)
+                lsps_[i] = lsps[i];
+            
             #ifdef DUMP
 	    dump_ak(ak, order);
             dump_E(e);
             #endif
-	
+
 	    /* tracking down -ve energy values with BW expansion */
 	    /*
 	    if (e < 0.0) {
 		int i;
 		FILE*f=fopen("x.txt","wt");
-		for(i=0; i<M; i++)
+		for(i=0; i<M_PITCH; i++)
 		    fprintf(f,"%f\n", Sn[i]);
 		fclose(f);
 		printf("e = %f frames = %d\n", e, frames);
@@ -588,6 +706,18 @@ int main(int argc, char *argv[])
 	    if (dump_pitch_e)
 		fprintf(fjvm, "%f\n", e);
 
+            /* speech centred on analysis frame for Deep Learning work */
+            
+            if (ten_ms_centre) {
+                int n_10_ms = Fs*0.01;
+                int n_5_ms = Fs*0.005;
+                short buf[n_10_ms];
+                for(i=0; i<n_10_ms; i++) {
+                    buf[i] = Sn[m_pitch/2-n_5_ms+i];
+                }
+                fwrite(buf, n_10_ms, sizeof(short), ften_ms_centre);                  
+            }
+            
             #ifdef DUMP
             dump_lsp(lsps);
             #endif
@@ -618,10 +748,10 @@ int main(int argc, char *argv[])
 	    if (lspjvm) {
 		/* Jean-Marc's multi-stage, split VQ */
 		lspjvm_quantise(lsps, lsps_, LPC_ORD);
-		{ 
+		{
 		    float lsps_bw[LPC_ORD];
-		    memcpy(lsps_bw, lsps_, sizeof(float)*LPC_ORD);
-		    bw_expand_lsps(lsps_bw, LPC_ORD, 50.0, 100.0);			    
+		    memcpy(lsps_bw, lsps_, sizeof(float)*order);
+		    bw_expand_lsps(lsps_bw, LPC_ORD, 50.0, 100.0);
 		    lsp_to_lpc(lsps_bw, ak, LPC_ORD);
 		}
 	    }
@@ -631,7 +761,7 @@ int main(int argc, char *argv[])
 		/*  multi-stage VQ from Anssi Ramo OH3GDD */
 
 		lspanssi_quantise(lsps, lsps_, LPC_ORD, 5);
-		bw_expand_lsps(lsps_, LPC_ORD, 50.0, 100.0);			    
+		bw_expand_lsps(lsps_, LPC_ORD, 50.0, 100.0);
 		lsp_to_lpc(lsps_, ak, LPC_ORD);
 	    }
 #endif
@@ -661,7 +791,7 @@ int main(int argc, char *argv[])
 		    f = (4000.0/PI)*lsps[i];
 		    mel[i] = floor(2595.0*log10(1.0 + f/700.0) + 0.5);
 		}
-                
+
                 #define MEL_ROUND 25
 		for(i=1; i<order; i++) {
 		    if (mel[i] <= mel[i-1]+MEL_ROUND) {
@@ -674,15 +804,15 @@ int main(int argc, char *argv[])
                 #ifdef DUMP
                 dump_mel(mel, order);
                 #endif
-                
+
  		encode_mels_scalar(mel_indexes, mel, 6);
                 #ifdef DUMP
                 dump_mel_indexes(mel_indexes, 6);
                 #endif
 		//decode_mels_scalar(mel, mel_indexes, 6);
-                
+
                 /* read in VQed lsp-mels from octave/melvq.m */
-           
+
                 if (lspmelread) {
                     float mel_[order];
                     int ret = fread(mel_, sizeof(float), order, flspmel);
@@ -692,15 +822,15 @@ int main(int argc, char *argv[])
                         mel[i] = mel_[i];
                     }
                 }
-                
+
                 if (lspmelvq) {
                     int indexes[3];
                     //lspmelvq_mse += lspmelvq_quantise(mel, mel, order);
                     lspmelvq_mse += lspmelvq_mbest_encode(indexes, mel, mel, order, 5);
                 }
-        
+
                 /* ensure no unstable filters after quantisation */
-                       
+
                 #define MEL_ROUND 25
 		for(i=1; i<order; i++) {
 		    if (mel[i] <= mel[i-1]+MEL_ROUND) {
@@ -709,37 +839,37 @@ int main(int argc, char *argv[])
                         i = 1;
                     }
 		}
-                
+
 		for(i=0; i<order; i++) {
 		    f_ = 700.0*( pow(10.0, mel[i]/2595.0) - 1.0);
 		    lsps_[i] = f_*(PI/4000.0);
 		}
-                
+
 		lsp_to_lpc(lsps_, ak, order);
-                
+
 	    }
-                
+
 	    if (scalar_quant_Wo_e) {
 
 		e = decode_energy(encode_energy(e, E_BITS), E_BITS);
-                model.Wo = decode_Wo(encode_Wo(model.Wo, WO_BITS), WO_BITS);
+                model.Wo = decode_Wo(&c2const, encode_Wo(&c2const, model.Wo, WO_BITS), WO_BITS);
 		model.L  = PI/model.Wo; /* if we quantise Wo re-compute L */
 	    }
-                
+
 	    if (scalar_quant_Wo_e_low) {
                 int ind;
 		e = decode_energy(ind = encode_energy(e, 3), 3);
-                model.Wo = decode_log_Wo(encode_log_Wo(model.Wo, 5), 5);
+                model.Wo = decode_log_Wo(&c2const, encode_log_Wo(&c2const, model.Wo, 5), 5);
 		model.L  = PI/model.Wo; /* if we quantise Wo re-compute L */
 	    }
-                
+
 	    if (vector_quant_Wo_e) {
 
 		/* JVM's experimental joint Wo & LPC energy quantiser */
 
-		quantise_WoE(&model, &e, Woe_);
+		quantise_WoE(&c2const, &model, &e, Woe_);
 	    }
-            
+
 	}
 
         if (amread) {
@@ -747,13 +877,35 @@ int main(int argc, char *argv[])
             assert(ret == MAX_AMP);
         }
 
+        if (Woread) {
+            int ret = fread(&model.Wo, sizeof(float), 1, fWo);
+            model.L = floor(PI/model.Wo);
+            assert(ret == 1);
+        }
+
+        /* dump features for Deep learning, placed here so we can get quantised features */
+        
+        if (lspEWov) {
+            /* order LSPs - energy - Wo - voicing flag - order LPCs */                
+            if (lsp)
+                fwrite(lsps_, order, sizeof(float), flspEWov);
+            else
+                fwrite(lsps, order, sizeof(float), flspEWov);
+                    
+            fwrite(&e, 1, sizeof(float), flspEWov);
+            fwrite(&model.Wo, 1, sizeof(float), flspEWov); 
+            float voiced_float = model.voiced;
+            fwrite(&voiced_float, 1, sizeof(float), flspEWov);
+            fwrite(&ak[1], order, sizeof(float), flspEWov);
+        }
+            
 	/*------------------------------------------------------------*\
 
           Synthesise and optional decimation to 20 or 40ms frame rate
 
 	\*------------------------------------------------------------*/
 
-        /* 
+        /*
            if decimate == 2, we interpolate frame n from frame n-1 and n+1
            if decimate == 4, we interpolate frames n, n+1, n+2, from frames n-1 and n+3
 
@@ -766,7 +918,7 @@ int main(int argc, char *argv[])
         for(i=0; i<decimate-1; i++)
             model_dec[i] = model_dec[i+1];
         model_dec[decimate-1] = model;
-                
+
         if ((frames % decimate) == 0) {
             for(i=0; i<order; i++)
                 lsps_dec[decimate-1][i] = lsps_[i];
@@ -779,16 +931,16 @@ int main(int argc, char *argv[])
             for(i=0, weight=weight_inc; i<decimate-1; i++, weight += weight_inc) {
                 //model_dec[i].voiced = model_dec[decimate-1].voiced;
                 interpolate_lsp_ver2(&lsps_dec[i][0], prev_lsps_dec, &lsps_dec[decimate-1][0], weight, order);
-                interp_Wo2(&model_dec[i], &prev_model_dec, &model_dec[decimate-1], weight);
+                interp_Wo2(&model_dec[i], &prev_model_dec, &model_dec[decimate-1], weight, c2const.Wo_min);
                 e_dec[i] = interp_energy2(prev_e_dec, e_dec[decimate-1],weight);
             }
 
-            /* then recover spectral amplitudes and synthesise */    
+            /* then recover spectral amplitudes and synthesise */
 
             for(i=0; i<decimate; i++) {
                 if (lpc_model) {
-                    lsp_to_lpc(&lsps_dec[i][0], &ak_dec[i][0], order);                
-                    aks_to_M2(fft_fwd_cfg, &ak_dec[i][0], order, &model_dec[i], e_dec[i], 
+                    lsp_to_lpc(&lsps_dec[i][0], &ak_dec[i][0], order);
+                    aks_to_M2(fftr_fwd_cfg, &ak_dec[i][0], order, &model_dec[i], e_dec[i],
                               &snr, 0, simlpcpf, lpcpf, 1, LPCPF_BETA, LPCPF_GAMMA, Aw);
                     apply_lpc_correction(&model_dec[i]);
                     sum_snr += snr;
@@ -799,12 +951,30 @@ int main(int argc, char *argv[])
                     #endif
                 }
 
-                if (phase0)
-                    phase_synth_zero_order(fft_fwd_cfg, &model_dec[i], ex_phase, Aw);	
+                if (phase0) {
+                    /* optionally read in Aw, replacing values generated using LPC */
+
+                    if (awread) {
+                        int ret = fread(Aw, sizeof(COMP), FFT_ENC, faw);
+                        assert(ret == FFT_ENC);
+                    }
+                    
+                    /* optionally read in Hm directly, bypassing sampling of Aw[] */
+
+                    if (hmread) {
+                        int ret = fread(H, sizeof(COMP), MAX_AMP, fhm);
+                        assert(ret == MAX_AMP);
+                    } else {
+                        determine_phase(&c2const, H, &model_dec[i], NEWAMP1_PHASE_NFFT, phase_fft_fwd_cfg, phase_fft_inv_cfg);
+                    }
+                    phase_synth_zero_order(n_samp, &model_dec[i], ex_phase, H);
+                }
+
+                
                 if (postfilt)
                     postfilter(&model_dec[i], &bg_est);
-                synth_one_frame(fft_inv_cfg, buf, &model_dec[i], Sn_, Pn, prede, &de_mem, gain);
-                if (fout != NULL) fwrite(buf,sizeof(short),N,fout);
+                synth_one_frame(n_samp, fftr_inv_cfg, buf, &model_dec[i], Sn_, Pn, prede, &de_mem, gain);
+                if (fout != NULL) fwrite(buf,sizeof(short),N_SAMP,fout);
             }
 
             /* update memories for next frame ----------------------------*/
@@ -812,11 +982,11 @@ int main(int argc, char *argv[])
             prev_model_dec = model_dec[decimate-1];
             prev_e_dec = e_dec[decimate-1];
             for(i=0; i<LPC_ORD; i++)
-                prev_lsps_dec[i] = lsps_dec[decimate-1][i];        
+                prev_lsps_dec[i] = lsps_dec[decimate-1][i];
        }
 
     }
-                
+
     /*----------------------------------------------------------------*\
 
                             End Main Loop
@@ -824,7 +994,7 @@ int main(int argc, char *argv[])
     \*----------------------------------------------------------------*/
 
     fclose(fin);
-                
+
     if (fout != NULL)
 	fclose(fout);
 
@@ -834,10 +1004,12 @@ int main(int argc, char *argv[])
             fprintf(stderr, "lspmelvq std = %3.1f Hz\n", sqrt(lspmelvq_mse/frames));
     }
 
+    #if 0
     if (phaseexp)
 	phase_experiment_destroy(pexp);
     if (ampexp)
 	amp_experiment_destroy(aexp);
+    #endif
     #ifdef DUMP
     if (dump)
 	dump_off();
@@ -847,20 +1019,29 @@ int main(int argc, char *argv[])
 	fclose(fvoicing);
 
     nlp_destroy(nlp_states);
+
+    if (flspmel != NULL) fclose(flspmel);
+    if (fam     != NULL) fclose(fam);
+    if (fWo     != NULL) fclose(fWo);
+    if (faw     != NULL) fclose(faw);
+    if (fhm     != NULL) fclose(fhm);
+    if (fjvm    != NULL) fclose(fjvm);
+    if (flspEWov != NULL) fclose(flspEWov);
+    if (ften_ms_centre != NULL) fclose(ften_ms_centre);
     
     return 0;
 }
-                
-void synth_one_frame(kiss_fft_cfg fft_inv_cfg, short buf[], MODEL *model, float Sn_[], 
+
+void synth_one_frame(int n_samp, codec2_fftr_cfg fftr_inv_cfg, short buf[], MODEL *model, float Sn_[],
                      float Pn[], int prede, float *de_mem, float gain)
 {
     int     i;
 
-    synthesise(fft_inv_cfg, Sn_, model, Pn, 1);
+    synthesise(n_samp, fftr_inv_cfg, Sn_, model, Pn, 1);
     if (prede)
-        de_emp(Sn_, Sn_, de_mem, N);	
+        de_emp(Sn_, Sn_, de_mem, n_samp);
 
-    for(i=0; i<N; i++) {
+    for(i=0; i<n_samp; i++) {
 	Sn_[i] *= gain;
 	if (Sn_[i] > 32767.0)
 	    buf[i] = 32767;
