@@ -30,8 +30,17 @@
 
 #define DAC_BUF_SZ   320
 
-void dac_open(int fifo_sz);
-int dac1_write(short buf[], int n); /* DAC1 pin PA4 */
-int dac2_write(short buf[], int n); /* DAC2 pin PA5 */
+/* divisors for various sample rates */
+
+#define DAC_FS_8KHZ 10500
+#define DAC_FS_16KHZ 5250
+#define DAC_FS_48KHZ 1750
+#define DAC_FS_96KHZ 875
+
+void dac_open(int fs_divisor, int fifo_sz, short *buf1, short *buf2);
+int dac1_write(short buf[], int n, int limit); /* DAC1 pin PA4 */
+int dac1_free();
+int dac2_write(short buf[], int n, int limit); /* DAC2 pin PA5 */
+int dac2_free();
 
 #endif
