@@ -4,21 +4,21 @@
   * @author  MCD Application Team
   * @version V1.0.0
   * @date    22-July-2011
-  * @brief   This file provides the high layer firmware functions to manage the 
+  * @brief   This file provides the high layer firmware functions to manage the
   *          following functionalities of the USB CDC Class:
   *           - Initialization and Configuration of high and low layer
   *           - Enumeration as CDC Device (and enumeration for each implemented memory interface)
   *           - OUT/IN data transfer
   *           - Command IN transfer (class requests management)
   *           - Error management
-  *           
+  *
   *  @verbatim
-  *      
-  *          ===================================================================      
+  *
+  *          ===================================================================
   *                                CDC Class Driver Description
-  *          =================================================================== 
+  *          ===================================================================
   *           This driver manages the "Universal Serial Bus Class Definitions for Communications Devices
-  *           Revision 1.2 November 16, 2007" and the sub-protocol specification of "Universal Serial Bus 
+  *           Revision 1.2 November 16, 2007" and the sub-protocol specification of "Universal Serial Bus
   *           Communications Class Subclass Specification for PSTN Devices Revision 1.2 February 9, 2007"
   *           This driver implements the following aspects of the specification:
   *             - Device descriptor management
@@ -33,17 +33,17 @@
   *             For the Abstract Control Model, this core allows only transmitting the requests to
   *             lower layer dispatcher (ie. usbd_cdc_vcp.c/.h) which should manage each request and
   *             perform relative actions.
-  * 
+  *
   *           These aspects may be enriched or modified for a specific user application.
-  *          
-  *            This driver doesn't implement the following aspects of the specification 
+  *
+  *            This driver doesn't implement the following aspects of the specification
   *            (but it is possible to manage these features with some modifications on this driver):
   *             - Any class-specific aspect relative to communication classes should be managed by user application.
   *             - All communication classes other than PSTN are not managed
-  *      
+  *
   *  @endverbatim
-  *                                  
-  ******************************************************************************               
+  *
+  ******************************************************************************
   * @attention
   *
   * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
@@ -55,7 +55,7 @@
   *
   * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
   ******************************************************************************
-  */ 
+  */
 
 /* Includes ------------------------------------------------------------------*/
 #include "usbd_cdc_core.h"
@@ -68,33 +68,33 @@
   */
 
 
-/** @defgroup usbd_cdc 
+/** @defgroup usbd_cdc
   * @brief usbd core module
   * @{
-  */ 
+  */
 
 /** @defgroup usbd_cdc_Private_TypesDefinitions
   * @{
-  */ 
+  */
 /**
   * @}
-  */ 
+  */
 
 
 /** @defgroup usbd_cdc_Private_Defines
   * @{
-  */ 
+  */
 /**
   * @}
-  */ 
+  */
 
 
 /** @defgroup usbd_cdc_Private_Macros
   * @{
-  */ 
+  */
 /**
   * @}
-  */ 
+  */
 
 
 /** @defgroup usbd_cdc_Private_FunctionPrototypes
@@ -117,58 +117,58 @@ static uint8_t  usbd_cdc_SOF         (void *pdev);
  *********************************************/
 static void Handle_USBAsynchXfer  (void *pdev);
 static uint8_t  *USBD_cdc_GetCfgDesc (uint8_t speed, uint16_t *length);
-#ifdef USE_USB_OTG_HS  
+#ifdef USE_USB_OTG_HS
 static uint8_t  *USBD_cdc_GetOtherCfgDesc (uint8_t speed, uint16_t *length);
 #endif
 /**
   * @}
-  */ 
+  */
 
 /** @defgroup usbd_cdc_Private_Variables
   * @{
-  */ 
+  */
 extern CDC_IF_Prop_TypeDef  APP_FOPS;
 extern uint8_t USBD_DeviceDesc   [USB_SIZ_DEVICE_DESC];
 
 #ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
   #if defined ( __ICCARM__ ) /*!< IAR Compiler */
-    #pragma data_alignment=4   
+    #pragma data_alignment=4
   #endif
 #endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */
 __ALIGN_BEGIN uint8_t usbd_cdc_CfgDesc  [USB_CDC_CONFIG_DESC_SIZ] __ALIGN_END ;
 
 #ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
   #if defined ( __ICCARM__ ) /*!< IAR Compiler */
-    #pragma data_alignment=4   
+    #pragma data_alignment=4
   #endif
 #endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */
 __ALIGN_BEGIN uint8_t usbd_cdc_OtherCfgDesc  [USB_CDC_CONFIG_DESC_SIZ] __ALIGN_END ;
 
 #ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
   #if defined ( __ICCARM__ ) /*!< IAR Compiler */
-    #pragma data_alignment=4   
+    #pragma data_alignment=4
   #endif
 #endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */
 __ALIGN_BEGIN static __IO uint32_t  usbd_cdc_AltSet  __ALIGN_END = 0;
 
 #ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
   #if defined ( __ICCARM__ ) /*!< IAR Compiler */
-    #pragma data_alignment=4   
+    #pragma data_alignment=4
   #endif
 #endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */
 __ALIGN_BEGIN uint8_t USB_Rx_Buffer   [CDC_DATA_MAX_PACKET_SIZE] __ALIGN_END ;
 
 #ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
   #if defined ( __ICCARM__ ) /*!< IAR Compiler */
-    #pragma data_alignment=4   
+    #pragma data_alignment=4
   #endif
 #endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */
-__ALIGN_BEGIN uint8_t APP_Rx_Buffer   [APP_RX_DATA_SIZE] __ALIGN_END ; 
+__ALIGN_BEGIN uint8_t APP_Rx_Buffer   [APP_RX_DATA_SIZE] __ALIGN_END ;
 
 
 #ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
   #if defined ( __ICCARM__ ) /*!< IAR Compiler */
-    #pragma data_alignment=4   
+    #pragma data_alignment=4
   #endif
 #endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */
 __ALIGN_BEGIN uint8_t CmdBuff[CDC_CMD_PACKET_SZE] __ALIGN_END ;
@@ -183,7 +183,7 @@ static uint32_t cdcCmd = 0xFF;
 static uint32_t cdcLen = 0;
 
 /* CDC interface class callbacks structure */
-USBD_Class_cb_TypeDef  USBD_CDC_cb = 
+USBD_Class_cb_TypeDef  USBD_CDC_cb =
 {
   usbd_cdc_Init,
   usbd_cdc_DeInit,
@@ -194,16 +194,16 @@ USBD_Class_cb_TypeDef  USBD_CDC_cb =
   usbd_cdc_DataOut,
   usbd_cdc_SOF,
   NULL,
-  NULL,     
+  NULL,
   USBD_cdc_GetCfgDesc,
-#ifdef USE_USB_OTG_HS   
+#ifdef USE_USB_OTG_HS
   USBD_cdc_GetOtherCfgDesc, /* use same cobfig as per FS */
 #endif /* USE_USB_OTG_HS  */
 };
 
 #ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
   #if defined ( __ICCARM__ ) /*!< IAR Compiler */
-    #pragma data_alignment=4   
+    #pragma data_alignment=4
   #endif
 #endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */
 /* USB CDC device Configuration Descriptor */
@@ -219,9 +219,9 @@ __ALIGN_BEGIN uint8_t usbd_cdc_CfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_END =
   0x00,   /* iConfiguration: Index of string descriptor describing the configuration */
   0xC0,   /* bmAttributes: self powered */
   0x32,   /* MaxPower 0 mA */
-  
+
   /*---------------------------------------------------------------------------*/
-  
+
   /*Interface Descriptor */
   0x09,   /* bLength: Interface Descriptor size */
   USB_INTERFACE_DESCRIPTOR_TYPE,  /* bDescriptorType: Interface */
@@ -233,34 +233,34 @@ __ALIGN_BEGIN uint8_t usbd_cdc_CfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_END =
   0x02,   /* bInterfaceSubClass: Abstract Control Model */
   0x01,   /* bInterfaceProtocol: Common AT commands */
   0x00,   /* iInterface: */
-  
+
   /*Header Functional Descriptor*/
   0x05,   /* bLength: Endpoint Descriptor size */
   0x24,   /* bDescriptorType: CS_INTERFACE */
   0x00,   /* bDescriptorSubtype: Header Func Desc */
   0x10,   /* bcdCDC: spec release number */
   0x01,
-  
+
   /*Call Management Functional Descriptor*/
   0x05,   /* bFunctionLength */
   0x24,   /* bDescriptorType: CS_INTERFACE */
   0x01,   /* bDescriptorSubtype: Call Management Func Desc */
   0x00,   /* bmCapabilities: D0+D1 */
   0x01,   /* bDataInterface: 1 */
-  
+
   /*ACM Functional Descriptor*/
   0x04,   /* bFunctionLength */
   0x24,   /* bDescriptorType: CS_INTERFACE */
   0x02,   /* bDescriptorSubtype: Abstract Control Management desc */
   0x02,   /* bmCapabilities */
-  
+
   /*Union Functional Descriptor*/
   0x05,   /* bFunctionLength */
   0x24,   /* bDescriptorType: CS_INTERFACE */
   0x06,   /* bDescriptorSubtype: Union func desc */
   0x00,   /* bMasterInterface: Communication class interface */
   0x01,   /* bSlaveInterface0: Data Class Interface */
-  
+
   /*Endpoint 2 Descriptor*/
   0x07,                           /* bLength: Endpoint Descriptor size */
   USB_ENDPOINT_DESCRIPTOR_TYPE,   /* bDescriptorType: Endpoint */
@@ -273,9 +273,9 @@ __ALIGN_BEGIN uint8_t usbd_cdc_CfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_END =
 #else
   0xFF,                           /* bInterval: */
 #endif /* USE_USB_OTG_HS */
-  
+
   /*---------------------------------------------------------------------------*/
-  
+
   /*Data class interface descriptor*/
   0x09,   /* bLength: Endpoint Descriptor size */
   USB_INTERFACE_DESCRIPTOR_TYPE,  /* bDescriptorType: */
@@ -286,7 +286,7 @@ __ALIGN_BEGIN uint8_t usbd_cdc_CfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_END =
   0x00,   /* bInterfaceSubClass: */
   0x00,   /* bInterfaceProtocol: */
   0x00,   /* iInterface: */
-  
+
   /*Endpoint OUT Descriptor*/
   0x07,   /* bLength: Endpoint Descriptor size */
   USB_ENDPOINT_DESCRIPTOR_TYPE,      /* bDescriptorType: Endpoint */
@@ -295,7 +295,7 @@ __ALIGN_BEGIN uint8_t usbd_cdc_CfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_END =
   LOBYTE(CDC_DATA_MAX_PACKET_SIZE),  /* wMaxPacketSize: */
   HIBYTE(CDC_DATA_MAX_PACKET_SIZE),
   0x00,                              /* bInterval: ignore for Bulk transfer */
-  
+
   /*Endpoint IN Descriptor*/
   0x07,   /* bLength: Endpoint Descriptor size */
   USB_ENDPOINT_DESCRIPTOR_TYPE,      /* bDescriptorType: Endpoint */
@@ -309,21 +309,21 @@ __ALIGN_BEGIN uint8_t usbd_cdc_CfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_END =
 #ifdef USE_USB_OTG_HS
 #ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
   #if defined ( __ICCARM__ ) /*!< IAR Compiler */
-    #pragma data_alignment=4   
+    #pragma data_alignment=4
   #endif
-#endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */ 
+#endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */
 __ALIGN_BEGIN uint8_t usbd_cdc_OtherCfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_END =
-{ 
+{
   0x09,   /* bLength: Configuation Descriptor size */
-  USB_DESC_TYPE_OTHER_SPEED_CONFIGURATION,   
+  USB_DESC_TYPE_OTHER_SPEED_CONFIGURATION,
   USB_CDC_CONFIG_DESC_SIZ,
   0x00,
   0x02,   /* bNumInterfaces: 2 interfaces */
   0x01,   /* bConfigurationValue: */
   0x04,   /* iConfiguration: */
   0xC0,   /* bmAttributes: */
-  0x32,   /* MaxPower 100 mA */  
-  
+  0x32,   /* MaxPower 100 mA */
+
   /*Interface Descriptor */
   0x09,   /* bLength: Interface Descriptor size */
   USB_INTERFACE_DESCRIPTOR_TYPE,  /* bDescriptorType: Interface */
@@ -335,34 +335,34 @@ __ALIGN_BEGIN uint8_t usbd_cdc_OtherCfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_EN
   0x02,   /* bInterfaceSubClass: Abstract Control Model */
   0x01,   /* bInterfaceProtocol: Common AT commands */
   0x00,   /* iInterface: */
-  
+
   /*Header Functional Descriptor*/
   0x05,   /* bLength: Endpoint Descriptor size */
   0x24,   /* bDescriptorType: CS_INTERFACE */
   0x00,   /* bDescriptorSubtype: Header Func Desc */
   0x10,   /* bcdCDC: spec release number */
   0x01,
-  
+
   /*Call Management Functional Descriptor*/
   0x05,   /* bFunctionLength */
   0x24,   /* bDescriptorType: CS_INTERFACE */
   0x01,   /* bDescriptorSubtype: Call Management Func Desc */
   0x00,   /* bmCapabilities: D0+D1 */
   0x01,   /* bDataInterface: 1 */
-  
+
   /*ACM Functional Descriptor*/
   0x04,   /* bFunctionLength */
   0x24,   /* bDescriptorType: CS_INTERFACE */
   0x02,   /* bDescriptorSubtype: Abstract Control Management desc */
   0x02,   /* bmCapabilities */
-  
+
   /*Union Functional Descriptor*/
   0x05,   /* bFunctionLength */
   0x24,   /* bDescriptorType: CS_INTERFACE */
   0x06,   /* bDescriptorSubtype: Union func desc */
   0x00,   /* bMasterInterface: Communication class interface */
   0x01,   /* bSlaveInterface0: Data Class Interface */
-  
+
   /*Endpoint 2 Descriptor*/
   0x07,                           /* bLength: Endpoint Descriptor size */
   USB_ENDPOINT_DESCRIPTOR_TYPE,   /* bDescriptorType: Endpoint */
@@ -371,9 +371,9 @@ __ALIGN_BEGIN uint8_t usbd_cdc_OtherCfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_EN
   LOBYTE(CDC_CMD_PACKET_SZE),     /* wMaxPacketSize: */
   HIBYTE(CDC_CMD_PACKET_SZE),
   0xFF,                           /* bInterval: */
-  
+
   /*---------------------------------------------------------------------------*/
-  
+
   /*Data class interface descriptor*/
   0x09,   /* bLength: Endpoint Descriptor size */
   USB_INTERFACE_DESCRIPTOR_TYPE,  /* bDescriptorType: */
@@ -384,7 +384,7 @@ __ALIGN_BEGIN uint8_t usbd_cdc_OtherCfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_EN
   0x00,   /* bInterfaceSubClass: */
   0x00,   /* bInterfaceProtocol: */
   0x00,   /* iInterface: */
-  
+
   /*Endpoint OUT Descriptor*/
   0x07,   /* bLength: Endpoint Descriptor size */
   USB_ENDPOINT_DESCRIPTOR_TYPE,      /* bDescriptorType: Endpoint */
@@ -393,7 +393,7 @@ __ALIGN_BEGIN uint8_t usbd_cdc_OtherCfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_EN
   0x40,                              /* wMaxPacketSize: */
   0x00,
   0x00,                              /* bInterval: ignore for Bulk transfer */
-  
+
   /*Endpoint IN Descriptor*/
   0x07,   /* bLength: Endpoint Descriptor size */
   USB_ENDPOINT_DESCRIPTOR_TYPE,     /* bDescriptorType: Endpoint */
@@ -407,11 +407,11 @@ __ALIGN_BEGIN uint8_t usbd_cdc_OtherCfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_EN
 
 /**
   * @}
-  */ 
+  */
 
 /** @defgroup usbd_cdc_Private_Functions
   * @{
-  */ 
+  */
 
 /**
   * @brief  usbd_cdc_Init
@@ -420,7 +420,7 @@ __ALIGN_BEGIN uint8_t usbd_cdc_OtherCfgDesc[USB_CDC_CONFIG_DESC_SIZ]  __ALIGN_EN
   * @param  cfgidx: Configuration index
   * @retval status
   */
-static uint8_t  usbd_cdc_Init (void  *pdev, 
+static uint8_t  usbd_cdc_Init (void  *pdev,
                                uint8_t cfgidx)
 {
   uint8_t *pbuf;
@@ -430,23 +430,23 @@ static uint8_t  usbd_cdc_Init (void  *pdev,
               CDC_IN_EP,
               CDC_DATA_IN_PACKET_SIZE,
               USB_OTG_EP_BULK);
-  
+
   /* Open EP OUT */
   DCD_EP_Open(pdev,
               CDC_OUT_EP,
               CDC_DATA_OUT_PACKET_SIZE,
               USB_OTG_EP_BULK);
-  
+
   /* Open Command IN EP */
   DCD_EP_Open(pdev,
               CDC_CMD_EP,
               CDC_CMD_PACKET_SZE,
               USB_OTG_EP_INT);
-  
+
   pbuf = (uint8_t *)USBD_DeviceDesc;
   pbuf[4] = DEVICE_CLASS_CDC;
   pbuf[5] = DEVICE_SUBCLASS_CDC;
-  
+
   /* Initialize the Interface physical components */
   APP_FOPS.pIf_Init();
 
@@ -455,7 +455,7 @@ static uint8_t  usbd_cdc_Init (void  *pdev,
                    CDC_OUT_EP,
                    (uint8_t*)(USB_Rx_Buffer),
                    CDC_DATA_OUT_PACKET_SIZE);
-  
+
   return USBD_OK;
 }
 
@@ -466,24 +466,24 @@ static uint8_t  usbd_cdc_Init (void  *pdev,
   * @param  cfgidx: Configuration index
   * @retval status
   */
-static uint8_t  usbd_cdc_DeInit (void  *pdev, 
+static uint8_t  usbd_cdc_DeInit (void  *pdev,
                                  uint8_t cfgidx)
 {
   /* Open EP IN */
   DCD_EP_Close(pdev,
               CDC_IN_EP);
-  
+
   /* Open EP OUT */
   DCD_EP_Close(pdev,
               CDC_OUT_EP);
-  
+
   /* Open Command IN EP */
   DCD_EP_Close(pdev,
               CDC_CMD_EP);
 
   /* Restore default state of the Interface physical components */
   APP_FOPS.pIf_DeInit();
-  
+
   return USBD_OK;
 }
 
@@ -494,12 +494,12 @@ static uint8_t  usbd_cdc_DeInit (void  *pdev,
   * @param  req: usb requests
   * @retval status
   */
-static uint8_t  usbd_cdc_Setup (void  *pdev, 
+static uint8_t  usbd_cdc_Setup (void  *pdev,
                                 USB_SETUP_REQ *req)
 {
-  uint16_t len;
+  uint16_t len = 0;
   uint8_t  *pbuf;
-  
+
   switch (req->bmRequest & USB_REQ_TYPE_MASK)
   {
     /* CDC Class Requests -------------------------------*/
@@ -512,24 +512,24 @@ static uint8_t  usbd_cdc_Setup (void  *pdev,
         {
           /* Get the data to be sent to Host from interface layer */
           APP_FOPS.pIf_Ctrl(req->bRequest, CmdBuff, req->wLength);
-          
+
           /* Send the data to the host */
-          USBD_CtlSendData (pdev, 
+          USBD_CtlSendData (pdev,
                             CmdBuff,
-                            req->wLength);          
+                            req->wLength);
         }
         else /* Host-to-Device requeset */
         {
           /* Set the value of the current command to be processed */
           cdcCmd = req->bRequest;
           cdcLen = req->wLength;
-          
+
           /* Prepare the reception of the buffer over EP0
-          Next step: the received data will be managed in usbd_cdc_EP0_TxSent() 
+          Next step: the received data will be managed in usbd_cdc_EP0_TxSent()
           function. */
           USBD_CtlPrepareRx (pdev,
                              CmdBuff,
-                             req->wLength);          
+                             req->wLength);
         }
       }
       else /* No Data request */
@@ -537,41 +537,41 @@ static uint8_t  usbd_cdc_Setup (void  *pdev,
         /* Transfer the command to the interface layer */
         APP_FOPS.pIf_Ctrl(req->bRequest, NULL, 0);
       }
-      
+
       return USBD_OK;
-      
+
     default:
       USBD_CtlError (pdev, req);
       return USBD_FAIL;
-    
-      
-      
+
+
+
     /* Standard Requests -------------------------------*/
   case USB_REQ_TYPE_STANDARD:
     switch (req->bRequest)
     {
-    case USB_REQ_GET_DESCRIPTOR: 
+    case USB_REQ_GET_DESCRIPTOR:
       if( (req->wValue >> 8) == CDC_DESCRIPTOR_TYPE)
       {
 #ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
-        pbuf = usbd_cdc_Desc;   
+        pbuf = usbd_cdc_Desc;
 #else
         pbuf = usbd_cdc_CfgDesc + 9 + (9 * USBD_ITF_MAX_NUM);
-#endif 
+#endif
         len = MIN(USB_CDC_DESC_SIZ , req->wLength);
       }
-      
-      USBD_CtlSendData (pdev, 
+
+      USBD_CtlSendData (pdev,
                         pbuf,
                         len);
       break;
-      
+
     case USB_REQ_GET_INTERFACE :
       USBD_CtlSendData (pdev,
                         (uint8_t *)&usbd_cdc_AltSet,
                         1);
       break;
-      
+
     case USB_REQ_SET_INTERFACE :
       if ((uint8_t)(req->wValue) < USBD_ITF_MAX_NUM)
       {
@@ -595,16 +595,16 @@ static uint8_t  usbd_cdc_Setup (void  *pdev,
   * @retval status
   */
 static uint8_t  usbd_cdc_EP0_RxReady (void  *pdev)
-{ 
+{
   if (cdcCmd != NO_CMD)
   {
     /* Process the data */
     APP_FOPS.pIf_Ctrl(cdcCmd, CmdBuff, cdcLen);
-    
+
     /* Reset the command variable to default value */
     cdcCmd = NO_CMD;
   }
-  
+
   return USBD_OK;
 }
 
@@ -622,36 +622,36 @@ static uint8_t  usbd_cdc_DataIn (void *pdev, uint8_t epnum)
 
   if (USB_Tx_State == 1)
   {
-    if (APP_Rx_length == 0) 
+    if (APP_Rx_length == 0)
     {
       USB_Tx_State = 0;
     }
-    else 
+    else
     {
       if (APP_Rx_length > CDC_DATA_IN_PACKET_SIZE){
         USB_Tx_ptr = APP_Rx_ptr_out;
         USB_Tx_length = CDC_DATA_IN_PACKET_SIZE;
-        
+
         APP_Rx_ptr_out += CDC_DATA_IN_PACKET_SIZE;
-        APP_Rx_length -= CDC_DATA_IN_PACKET_SIZE;    
+        APP_Rx_length -= CDC_DATA_IN_PACKET_SIZE;
       }
-      else 
+      else
       {
         USB_Tx_ptr = APP_Rx_ptr_out;
         USB_Tx_length = APP_Rx_length;
-        
+
         APP_Rx_ptr_out += APP_Rx_length;
         APP_Rx_length = 0;
       }
-      
+
       /* Prepare the available data buffer to be sent on IN endpoint */
       DCD_EP_Tx (pdev,
                  CDC_IN_EP,
                  (uint8_t*)&APP_Rx_Buffer[USB_Tx_ptr],
                  USB_Tx_length);
     }
-  }  
-  
+  }
+
   return USBD_OK;
 }
 
@@ -663,16 +663,16 @@ static uint8_t  usbd_cdc_DataIn (void *pdev, uint8_t epnum)
   * @retval status
   */
 static uint8_t  usbd_cdc_DataOut (void *pdev, uint8_t epnum)
-{      
+{
   uint16_t USB_Rx_Cnt;
-  
+
   /* Get the received data buffer and update the counter */
   USB_Rx_Cnt = ((USB_OTG_CORE_HANDLE*)pdev)->dev.out_ep[epnum].xfer_count;
-  
-  /* USB data will be immediately processed, this allow next USB traffic being 
+
+  /* USB data will be immediately processed, this allow next USB traffic being
      NAKed till the end of the application Xfer */
   APP_FOPS.pIf_DataRx(USB_Rx_Buffer, USB_Rx_Cnt);
-  
+
   /* Prepare Out endpoint to receive next packet */
   DCD_EP_PrepareRx(pdev,
                    CDC_OUT_EP,
@@ -690,18 +690,18 @@ static uint8_t  usbd_cdc_DataOut (void *pdev, uint8_t epnum)
   * @retval status
   */
 static uint8_t  usbd_cdc_SOF (void *pdev)
-{      
+{
   static uint32_t FrameCount = 0;
-  
+
   if (FrameCount++ == CDC_IN_FRAME_INTERVAL)
   {
     /* Reset the frame counter */
     FrameCount = 0;
-    
+
     /* Check the data to be sent through IN pipe */
     Handle_USBAsynchXfer(pdev);
   }
-  
+
   return USBD_OK;
 }
 
@@ -715,62 +715,62 @@ static void Handle_USBAsynchXfer (void *pdev)
 {
   uint16_t USB_Tx_ptr;
   uint16_t USB_Tx_length;
-  
+
   if(USB_Tx_State != 1)
   {
     if (APP_Rx_ptr_out == APP_RX_DATA_SIZE)
     {
       APP_Rx_ptr_out = 0;
     }
-    
-    if(APP_Rx_ptr_out == APP_Rx_ptr_in) 
+
+    if(APP_Rx_ptr_out == APP_Rx_ptr_in)
     {
-      USB_Tx_State = 0; 
+      USB_Tx_State = 0;
       return;
     }
-    
+
     if(APP_Rx_ptr_out > APP_Rx_ptr_in) /* rollback */
-    { 
+    {
       APP_Rx_length = APP_RX_DATA_SIZE - APP_Rx_ptr_out;
-    
+
     }
-    else 
+    else
     {
       APP_Rx_length = APP_Rx_ptr_in - APP_Rx_ptr_out;
-     
+
     }
 #ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
      APP_Rx_length &= ~0x03;
 #endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */
-    
+
     if (APP_Rx_length > CDC_DATA_IN_PACKET_SIZE)
     {
       USB_Tx_ptr = APP_Rx_ptr_out;
       USB_Tx_length = CDC_DATA_IN_PACKET_SIZE;
-      
-      APP_Rx_ptr_out += CDC_DATA_IN_PACKET_SIZE;	
+
+      APP_Rx_ptr_out += CDC_DATA_IN_PACKET_SIZE;
       APP_Rx_length -= CDC_DATA_IN_PACKET_SIZE;
     }
     else
     {
       USB_Tx_ptr = APP_Rx_ptr_out;
       USB_Tx_length = APP_Rx_length;
-      
+
       APP_Rx_ptr_out += APP_Rx_length;
       APP_Rx_length = 0;
     }
-    USB_Tx_State = 1; 
+    USB_Tx_State = 1;
 
     DCD_EP_Tx (pdev,
                CDC_IN_EP,
                (uint8_t*)&APP_Rx_Buffer[USB_Tx_ptr],
                USB_Tx_length);
-  }  
-  
+  }
+
 }
 
 /**
-  * @brief  USBD_cdc_GetCfgDesc 
+  * @brief  USBD_cdc_GetCfgDesc
   *         Return configuration descriptor
   * @param  speed : current device speed
   * @param  length : pointer data length
@@ -783,13 +783,13 @@ static uint8_t  *USBD_cdc_GetCfgDesc (uint8_t speed, uint16_t *length)
 }
 
 /**
-  * @brief  USBD_cdc_GetCfgDesc 
+  * @brief  USBD_cdc_GetCfgDesc
   *         Return configuration descriptor
   * @param  speed : current device speed
   * @param  length : pointer data length
   * @retval pointer to descriptor buffer
   */
-#ifdef USE_USB_OTG_HS 
+#ifdef USE_USB_OTG_HS
 static uint8_t  *USBD_cdc_GetOtherCfgDesc (uint8_t speed, uint16_t *length)
 {
   *length = sizeof (usbd_cdc_OtherCfgDesc);
@@ -798,14 +798,14 @@ static uint8_t  *USBD_cdc_GetOtherCfgDesc (uint8_t speed, uint16_t *length)
 #endif
 /**
   * @}
-  */ 
+  */
 
 /**
   * @}
-  */ 
+  */
 
 /**
   * @}
-  */ 
+  */
 
 /******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
